package core.repositories.timemanager;

import core.entities.User;
import core.entities.timemanager.BasicTaskSchedule;
import core.entities.timemanager.Task;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.CrudRepository;
import org.springframework.stereotype.Repository;

import javax.transaction.Transactional;
import java.time.LocalDate;
import java.util.List;
import java.util.Optional;

@Repository
public interface BasicTaskScheduleRepository extends CrudRepository<BasicTaskSchedule, Long> {

    @Modifying
    @Transactional
    @Query(value = "DELETE FROM BasicTaskSchedule bts WHERE bts.scheduleID = ?1")
    void deleteBasicTaskScheduleByID(long id);

    @Modifying
    @Transactional
    @Query(value = "DELETE FROM BasicTaskSchedule bts WHERE bts.task = ?1")
    void deleteAllByTask(Task task);

    @Query(value = "SELECT bts FROM BasicTaskSchedule bts WHERE bts.task.taskgroup.user = ?1")
    List<BasicTaskSchedule> findAllByUser(User user);

    @Query(value = "SELECT bts FROM BasicTaskSchedule bts WHERE bts.task.taskgroup.user = ?1 AND bts.scheduleDate = ?2")
    List<BasicTaskSchedule> findAllByUserAndDate(User user, LocalDate localDate);

    @Query(value = "SELECT bts FROM BasicTaskSchedule bts WHERE bts.task.taskgroup.user.username = ?1 AND bts.startTime is not null  and bts.finishedTime is null")
    Optional<BasicTaskSchedule> findActiveTaskSchedule(String username);

    List<BasicTaskSchedule> findAllByStartTimeIsNull();

    @Query(value = "SELECT bts FROM BasicTaskSchedule bts WHERE bts.task.taskgroup.user = ?1 AND bts.scheduleDate = ?2 AND bts.finishedTime IS NOT NULL")
    List<BasicTaskSchedule> findAllFinishedByUserAndDate(User user, LocalDate now);

    @Query(value = "SELECT bts FROM BasicTaskSchedule bts WHERE bts.task.taskgroup.user = ?1 AND bts.startTime IS NULL AND bts.finishedTime IS NULL")
    List<BasicTaskSchedule> findAllUnstartedSchedulesOfUser(User user);
}
